package example;

import java.util.ResourceBundle;

import labir.utils.IconCache;
import labir.utils.ToolBarCreater;
import labir.utils.interfaces.IToolBar;

import org.eclipse.swt.SWT;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.program.Program;

import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolItem;

public class ToolbarTest implements IToolBar
{
	private          ToolBar    toolBar; 
	private  static  IconCache  iconCache;	
	private final String[]  IMAGES   = {"cut.gif"    ,
                                        "copy.gif"   ,
                                        "delete.gif" ,
                                        "parent.gif" ,
                                        "paste.gif"  ,
                                        "print.gif"  ,
                                        "refresh.gif",
                                        "search.gif" ,
                                        "shell.gif"  ,
            		                    "file.gif"  };
	private final int    ICON_SHELL    = 8; 
	private  final  int  MENU_DROPDOWN = 0;
                                  // FORMAT - image.idx, image, resources, text, style
	private static String[][] TOOLBAR  = {
         {"", "shell.gif"  , null     , "", ToolBarCreater.STYLE_DROP_DOWN},
         {null},
	     {"", "cut.gif"    , "cut"    , "", ToolBarCreater.STYLE_PUSH},
	     {"", "copy.gif"   , "copy"   , "", ToolBarCreater.STYLE_PUSH}, 
	     {"", "paste.gif"  , "paste"  , "", ToolBarCreater.STYLE_PUSH}, 
	     {null},
	     {"", "refresh.gif", "refresh", "", ToolBarCreater.STYLE_PUSH},
         {"", "search.gif" , null     , "", ToolBarCreater.STYLE_PUSH}}; 
	private  final   int[]   CURSORS = {SWT.CURSOR_WAIT, SWT.CURSOR_HAND,
			                            SWT.CURSOR_ARROW, SWT.CURSOR_HELP};
	private  Menu menu;
	
 	private  ResourceBundle  resource = ResourceBundle.getBundle("resources/toolbar");
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 	/**
 	 * Конструктор класса
 	 * @param shell основное окно приложения
 	 */
	public ToolbarTest(final Shell shell)
	{
		// Кэш изображений
		iconCache = new IconCache();
		iconCache.setImagesDir("/images/");
		// Загрузка изображений
		iconCache.loadImages (shell.getDisplay(), IMAGES);
		iconCache.loadCursors(shell.getDisplay(), CURSORS);
		// Установка иконки приложения
		shell.setImage(iconCache.getImage(ICON_SHELL));
		// Определение идентификаторов кнопок панели инструментов
		for (int i = 0; i < TOOLBAR.length; i++) {
			if ((TOOLBAR[i][0] != null) && (TOOLBAR[i][1].length() > 0)) {
				for (int j = 0; j < IMAGES.length; j++) {
					if (IMAGES[j].equalsIgnoreCase(TOOLBAR[i][1])) {
						TOOLBAR[i][0] = String.valueOf(j);
						break;
					}
				}
			}
		}
		// Создание панели инструментов
		ToolBarCreater tbCreater = new ToolBarCreater(shell, this);
		// SWT.FLAT | SWT.HORIZONTAL  SWT.VERTICAL
		toolBar = tbCreater.getToolBar(resource, TOOLBAR, SWT.FLAT); 
		
		addDropdownMenu(shell);
		// Добавление в панель инструментов разделителя - SWT.SEPARATOR
		tbCreater.addToolItem (toolBar, null, null);
		// Добавление в панель инструментов кнопок для Word и Excel
		addProgramItem (shell, "test.doc");
		addProgramItem (shell, "test.xls");
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Метод создания выпадающего меню
	 * @param shell основное окно приложения
	 */
	private void addDropdownMenu(final Shell shell)
	{
		 // Создание раскрывающегося меню
		menu = new Menu(shell, SWT.POP_UP);
	    new MenuItem(menu, SWT.PUSH).setText("Открыть проект");
	    new MenuItem(menu, SWT.PUSH).setText("Закрыть проект");
	    new MenuItem(menu, SWT.SEPARATOR);
	    new MenuItem(menu, SWT.PUSH).setText("Выйти");
	    
	    final ToolItem itemDropDown = toolBar.getItem(MENU_DROPDOWN);
	    // Удаление слушателя
	    Listener listener = itemDropDown.getListeners(SWT.Selection)[0];
	    itemDropDown.removeListener(SWT.Selection, listener);
	    itemDropDown.addListener(SWT.Selection, new Listener() {
	    	public void handleEvent(Event event) {
	            Rectangle bounds = itemDropDown.getBounds();
	            Point point = toolBar.toDisplay(bounds.x, bounds.y + bounds.height);
	            menu.setLocation(point);
	            menu.setVisible(true);
	    	}
	    });	    
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Процедура добавления кнопок программ 
	 * @param shell окно
	 * @param fname имя файла
	 */
	private void addProgramItem (final Shell shell, final String fname)
	{
		String   typeString = null;
		Image    iconImage  = null;
		Program  program    = null;
		int dot = fname.lastIndexOf('.');
		if (dot != -1) {
			// Выделение расширения имени файла
			String extension = fname.substring(dot);
			// Определение программы
			program = Program.findProgram(extension);
			if (program != null) {
				typeString = program.getName();
				iconImage = iconCache.getProgramImage(program);
			}
		}
		// Создание элементов панели инструментов
		ToolBarCreater tbCreater = new ToolBarCreater(shell, this);
		tbCreater.addToolItem (toolBar, iconImage, typeString, SWT.CHECK);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Реализация интерфейса IToolBar
	 * @param toolbar - панель инструментов
	 * @param idx - идентификатор кнопки
	 * @see IToolBar
	 */
	@Override
	public void selectToolbarItem(final ToolBar toolbar, final int idx)
	{
		switch(idx) {
		case 0 : break;
		case 1 : tbCut() ; break;
		case 2 : tbCopy(); break;
		case 3 : tbPaste(); break;
		default : System.out.println ("ToolbarTest.selectToolbarItem() : idx = " + idx);
		}		
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void tbCut  () {}
	private void tbCopy () {}
	private void tbPaste() {}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public static void main (String [] args)
	{
		Display display = new Display();
		Shell   shell   = new Shell (display);
		shell.setText("SWT пример использования ToolBar");

		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		gridLayout.marginHeight = gridLayout.marginWidth = 0;
		shell.setLayout(gridLayout);
		new ToolbarTest(shell);

		shell.open ();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch ())
				display.sleep ();
		}
		iconCache.freeResources();
		display.dispose ();
	}
}
